/*
 * Created on Dec 11, 2003
 */
import java.awt.event.*;
import java.awt.event.ItemListener;

import javax.swing.*;

import poker.*;
import poker.util.*;

import java.net.*;
import java.io.*;
import java.util.*;

/**
 * A Simple example bot that can plug into Poki's Poker Academy
 *
 * As an example, the bot has one configuration option -- a check
 * box that, when activated, makes the bot always call.
 *
 * @author adavidson@biotools.com
 *
 * pre-flop : before any card are dealt on table
 * flop     : after 3 cards are on table
 * turn     : after the 4th card is on the table
 * river    : after the 5th card is on the table
 */
public class TieltBot
    implements Player {
  private static final String ALWAYS_CALL_MODE = "ALWAYS_CALL_MODE";

  private int ourSeat; // our seat for the current hand
  private Card c1, c2; // our hole cards
  private GameInfo gi; // general game information
  private Preferences prefs; // the configuration options for this bot
  private HandEvaluator he = new HandEvaluator();

  private DataOutputStream output;
  private BufferedReader input;
  private boolean logToLog = true;

  private static final String MSG_BOARDCARDS = "msgBoardCards";
  private static final String MSG_NEWGAME = "msgNewGame";
  private static final String MSG_GAMEINFO = "msgGameInfo";
  private static final String MSG_STAGEEVENT = "msgStageEvent";
  private static final String MSG_ACTIONEVENT = "msgActionEvent";
  private static final String MSG_GAMEOVEREVENT = "msgGameOverEvent";
  private static final String MSG_REQUESTACTION = "msgRequestAction";
  private static final String MSG_WINEVENT = "msgWinEvent";


  public TieltBot() throws Exception {
    log("Tielt bot constructor called:");
  }

  /**
   * This function sends a formatted message to Tielt.
   * It also appends the "\n" to signify the end of the message.
   *
   * @param name String
   * @param value String
   */
  public void sendCmd(TieltParam param) {
    try {
      output.writeBytes(param.paramList() + "\n");
      log(" -- Message sending to Tielt: " + param.paramList() + "\n");
    }
    catch (IOException e) {
      log(" ***Exception caught while sending message to Tielt in sendCmd():");
    }
  }

  public int waitForTieltAction() {
    int retval = -1;

    String value, act;
    try {
      value = input.readLine();
      act = value.substring(value.indexOf("=") + 1, value.indexOf(")"));
      retval = Integer.parseInt(act.trim());
      log("substring: " + act + " " + retval);
    }
    catch (IOException e) {
      log("Exception in parsing action from tielt");
      retval = -1;
    }

    return retval;
  }

  /**
   * If you implement the getSettingsPanel() method, your bot will display
   * the panel in the Opponent Settings Dialog.
   * @return a GUI for configuring your bot (optional)
   */
  public JPanel getSettingsPanel() {
    JPanel jp = new JPanel();
    final JCheckBox acMode = new JCheckBox("Always Call Mode",
                                           prefs.
                                           getBooleanPreference(ALWAYS_CALL_MODE));
    acMode.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        prefs.setPreference(ALWAYS_CALL_MODE, acMode.isSelected());
      }
    });
    jp.add(acMode);
    return jp;
  }

  /**
   * Get the current settings for this bot.
   */
  public Preferences getPreferences() {
    log("getPreferences() called:");
    return prefs;
  }

  /**
   * Load the current settings for this bot.
   */
  public void init(Preferences playerPrefs) {
    log("init() called:");
    this.prefs = playerPrefs;

    try {
      ServerSocket serverSock = new ServerSocket(5555);
      Socket connection = serverSock.accept();
      output = new DataOutputStream(connection.getOutputStream());
      input = new BufferedReader(new InputStreamReader(connection.
          getInputStream()));
    }
    catch (IOException e) {
      log("***Exception caught in init while trying to connect to client:");
    }
    log("Server accepted a connection:");
  }

  /**
   * An example setting for this bot. It can be turned into
   * an always-call mode, or to a simple strategy.
   * @return true if always-call mode is active.
   */
  public boolean getAlwaysCallMode() {
    System.out.println(" get always call mode:");
    return prefs.getBooleanPreference(ALWAYS_CALL_MODE, false);
  }

  /**
   * @return true if debug mode is on.
   */
  public boolean getDebug() {
    System.out.println(" getdebug()");
    return prefs.getBooleanPreference("DEBUG", true);
  }

  /**
   * print a debug statement.
   */
  public void debug(String str) {
    System.out.println(" getdebug( string ) ");
    if (getDebug()) {
      System.out.println(str);
    }
  }

  /**
   * print a debug statement with no end of line character
   */
  public void debugb(String str) {
    System.out.println(" getdebugb");
    if (getDebug()) {
      System.out.print(str);
    }
  }

  /**
   * Start playing a new game
   *
   * @param gInfo the current game information
   * @param c1 the first hole card
   * @param c2 the second hole card
   * @param seat the player's seat
   */
  public synchronized void newGame(GameInfo gInfo, Card c1, Card c2, int seat) {
    this.c1 = c1;
    this.c2 = c2;
    this.ourSeat = seat;
    this.gi = gInfo;

    TieltParam tp = new TieltParam(MSG_NEWGAME);

    tp.addParam("c1rank", c1.getRank());
    tp.addParam("c1suit", c1.getSuit());
    tp.addParam("c2rank", c2.getRank());
    tp.addParam("c2suit", c2.getSuit());
    tp.addParam("seat", seat);
    sendCmd(tp);

  }

  /**
   * An action has been observed.
   */
  public void actionEvent(int seat, int action, int amount) {
    log("actionEvent() called:");
    TieltParam tp = new TieltParam(MSG_ACTIONEVENT);
    tp.addParam("action", action);
    tp.addParam("amount", amount);
    tp.addParam("seat",seat);

    sendCmd(tp);

  }

  /**
   * The game is over
   */
  public void gameOverEvent() {
    log(" gameOverEvent called:");
    sendTieltGameInfo(); // send Tielt the game's current info

    TieltParam tp = new TieltParam(MSG_GAMEOVEREVENT);
    sendCmd(tp);

  }

  /**
   * A new betting round has started.
   */
  public void stageEvent(int stage) {
    log("stageEvent() called:");

    TieltParam tp = new TieltParam(MSG_STAGEEVENT);
    tp.addParam("Stage", stage);
    sendCmd(tp);

  }

  /**
   * A showdown has occurred.
   * @param pos the position of the player showing
   * @param c1 the first hole card shown
   * @param c2 the second hole card shown
   */
  public void showdownEvent(int seat, Card c1, Card c2) {
    log("showdownEvent called:");
  }

  /**
   * A new game has been started.
   * @param gi the game stat information
   */
  public void gameStartEvent(GameInfo gi) {
    log("gameStartEvent() called:");
  }

  /**
   * A player has won money.
   * @param pos the player position
   * @param amount the amount won
   * @param handName the name of the winning hand
   */
  public void winEvent(int seat, int amount, String handName) {
    log("winEvent called:");

    TieltParam tp = new TieltParam(MSG_WINEVENT);
    tp.addParam("amount", amount);
    tp.addParam("handName", handName);
    tp.addParam("seat", seat);

    sendCmd(tp);

  }

  /**
   * Get the players next action.
   *
   *
   * @return the player's action
   */
  public synchronized int action() {
    log("action() called ");
    int action = -1;

    sendTieltGameInfo(); // send Tielt the game's current info
    TieltParam tp = new TieltParam(MSG_REQUESTACTION);
    sendCmd(tp);

    action =  waitForTieltAction();
    System.out.println("value of action: " + action) ;

    if (action == -1) {
      if (gi.getAmountToCall(ourSeat) == 0){
        action = Holdem.CHECK;
      }
      else {
        action = Holdem.FOLD;
      }
    }

    return action;
  }



  public void sendTieltGameInfo() {
    int boardsize = gi.getBoard().size();
    Card tempCard;

    TieltParam tp = new TieltParam(MSG_GAMEINFO);
    tp.addParam("canRaise", gi.canRaise());
    tp.addParam("amountToCall", gi.getAmountToCall(ourSeat));
    tp.addParam("betSize", gi.getBetSize());
    tp.addParam("potSize", gi.getPot());
    tp.addParam("raiseAmount", gi.getRaiseAmount(ourSeat));
    tp.addParam("currentStage", gi.getStage());
    sendCmd(tp);

    tp = new TieltParam(MSG_BOARDCARDS);
    for (int i = 0; i < 5; i++) {
      if (i < boardsize) {
        tempCard = gi.getBoardCard(i);
        if (tempCard!=null){
          tp.addParam("bc" + Integer.toString(i + 1) + "rank", tempCard.getRank());
          tp.addParam("bc" + Integer.toString(i + 1) + "suit", tempCard.getSuit());
        }
      }
      else {
        tp.addParam("bc" + Integer.toString(i + 1) + "rank", -1);
        tp.addParam("bc" + Integer.toString(i + 1) + "suit", -1);
      }
    }
    sendCmd(tp);

  }

  // constants used in above method:
  //private final static int AHEAD = 0;
  //private final static int TIED = 1;
  //private final static int BEHIND = 2;

  public void log(String msg) {
    if (logToLog) System.out.println(msg);
  }

  /**
   * ------------------------------------------------------------------------------------------------
   */
  /**
   * This class provides a facility to stores the params of a message to send
   * to Tielt.  It will format the message in the right format before returning the string.
   */

  private class TieltParam {
    private String cmdName;
    private String retString;

    public TieltParam() {
      retString = "";
      cmdName = "noNameYet";
    }

    /**
     * Create a new TieltParam with a name for the param.
     * @param cmdName String
     */
    public TieltParam(String nameOfCmd) {
      retString = "";
      cmdName = nameOfCmd;
    }

    public String getParamName() {
      return cmdName;
    }

    public void setParamName(String pname) {
      cmdName = pname;
    }

    /**
     * Adding a param to the list of params
     * @param paramName String
     * @param paramValue String
     */
    public void addParam(String paramName, String paramValue) {
      if ( !retString.equalsIgnoreCase("") ){
        retString = retString + ", ";
      }
      retString = retString +paramName + " = " + paramValue;
    }

    public void addParam(String paramName, int paramValue) {
      if (!retString.equalsIgnoreCase("")) {
        retString = retString + ", ";
      }

      retString = retString + paramName + " = " + Integer.toString(paramValue);
    }

    public void addParam(String paramName, boolean paramValue) {
      String val;

      if (!retString.equalsIgnoreCase("")) {
        retString = retString + ", ";
      }

      if (paramValue) {
        val = "true";
      }
      else
        val = "false";
      retString = retString + paramName + " = " + val;
    }

    /**
     * This function takes the params and format them so they can be send
     * to Tielt.
     * @return String
     */
    public String paramList() {
      String retval = "";
      String p;

      if ( retString.equalsIgnoreCase("") ){
        retval = cmdName + "()";
      }
      else{
        retval = cmdName + "( " + retString + ")";;
      }
      return retval;
    }

  } //class TieltParam

}
